@@ -111,7 +111,6 @@ static gint               xfce_appfinder_model_item_compare_frecency  (gconstpoi
 static void               xfce_appfinder_model_frecency_collect       (XfceAppfinderModel       *model,
                                                                        GMappedFile              *mmap);
 static void               xfce_appfinder_model_frecency_free          (gpointer                  data);
-static gchar*             xfce_appfinder_model_unescape_entry_value   (const gchar              *value);
 static void               xfce_appfinder_model_migrate_history_file   (void);
 
 struct _XfceAppfinderModelClass
@@ -160,6 +159,8 @@ struct _XfceAppfinderModel
   XfceAppfinderIconSize  icon_size;
 
   gint                   scale_factor;
+
+  gboolean               generic_names;
 };
 
 typedef struct
@@ -207,7 +208,8 @@ enum
   PROP_0,
   PROP_ICON_SIZE,
   PROP_SCALE_FACTOR,
-  PROP_SORT_BY_FRECENCY
+  PROP_SORT_BY_FRECENCY,
+  PROP_GENERIC_NAMES
 };
 
 
@@ -251,6 +253,12 @@ xfce_appfinder_model_class_init (XfceAppfinderModelClass *klass)
                                                          FALSE,
                                                          G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_GENERIC_NAMES,
+                                   g_param_spec_boolean ("generic-names", NULL, NULL,
+                                                         FALSE,
+                                                         G_PARAM_READWRITE | G_PARAM_READWRITE));
+
   model_signals[CATEGORIES_CHANGED] =
     g_signal_new (g_intern_static_string ("categories-changed"),
                   G_TYPE_FROM_CLASS (gobject_class),
@@ -273,6 +281,7 @@ xfce_appfinder_model_init (XfceAppfinderModel *model)
   model->icon_size = XFCE_APPFINDER_ICON_SIZE_DEFAULT_ITEM;
   model->command_category = xfce_appfinder_model_get_command_category ();
   model->collect_cancelled = g_cancellable_new ();
+  model->generic_names = FALSE;
 
   model->menu = garcon_menu_new_applications ();
   appfinder_refcount_debug_add (G_OBJECT (model->menu), "main menu");
@@ -321,6 +330,10 @@ xfce_appfinder_model_get_property (GObject      *object,
       g_value_set_boolean (value, model->sort_by_frecency);
       break;
 
+    case PROP_GENERIC_NAMES:
+      g_value_set_boolean (value, model->generic_names);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -338,6 +351,7 @@ xfce_appfinder_model_set_property (GObject      *object,
   XfceAppfinderModel    *model = XFCE_APPFINDER_MODEL (object);
   XfceAppfinderIconSize  icon_size;
   gint                   scale_factor;
+  gboolean               generic_names;
 
   switch (prop_id)
     {
@@ -362,6 +376,15 @@ xfce_appfinder_model_set_property (GObject      *object,
       model->sort_by_frecency = g_value_get_boolean (value);
       break;
 
+    case PROP_GENERIC_NAMES:
+      generic_names = g_value_get_boolean (value);
+      if (model->generic_names != generic_names)
+        {
+          model->generic_names = generic_names;
+          xfce_appfinder_model_generic_names_changed (model);
+        }
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -517,6 +540,23 @@ xfce_appfinder_model_get_path (GtkTreeModel *tree_model,
 
 
 
+static const gchar*
+xfce_appfinder_model_get_menu_item_name (XfceAppfinderModel *model,
+                                         GarconMenuItem     *item)
+{
+  const gchar *name = NULL;
+
+  if (model->generic_names)
+    name = garcon_menu_item_get_generic_name (item);
+
+  if (name == NULL)
+    name = garcon_menu_item_get_name (item);
+
+  return name;
+}
+
+
+
 static void
 xfce_appfinder_model_get_value (GtkTreeModel *tree_model,
                                 GtkTreeIter  *iter,
@@ -549,7 +589,7 @@ xfce_appfinder_model_get_value (GtkTreeModel *tree_model,
         {
           if (item->item != NULL)
             {
-              name = garcon_menu_item_get_name (item->item);
+              name = xfce_appfinder_model_get_menu_item_name (model, item->item);
               comment = garcon_menu_item_get_comment (item->item);
 
               if (comment != NULL)
@@ -575,7 +615,10 @@ xfce_appfinder_model_get_value (GtkTreeModel *tree_model,
     case XFCE_APPFINDER_MODEL_COLUMN_TITLE:
       g_value_init (value, G_TYPE_STRING);
       if (item->item != NULL)
-        g_value_set_static_string (value, garcon_menu_item_get_name (item->item));
+        {
+          name = xfce_appfinder_model_get_menu_item_name (model, item->item);
+          g_value_set_static_string (value, name);
+        }
       else if (item->command != NULL)
         g_value_set_static_string (value, item->command);
       break;
@@ -2166,44 +2209,6 @@ xfce_appfinder_model_frecency_free (gpointer data)
 
 
 
-/* Remove this function when libxfce4util minimal required version is bumped to 4.18 */
-static gchar*
-xfce_appfinder_model_unescape_entry_value (const gchar *value)
-{
-  const gchar *p;
-  GString     *string;
-
-  if (G_UNLIKELY (value == NULL))
-    return NULL;
-
-  string = g_string_sized_new (strlen (value));
-
-  for (p = value; *p != '\0'; ++p)
-    {
-      if (G_UNLIKELY (p[0] == '\\' && p[1] != '\0'))
-        {
-          switch (*++p)
-            {
-            case 's':
-              g_string_append_c (string, ' ');
-              break;
-
-            case '\\':
-              g_string_append_c (string, '\\');
-              break;
-            }
-        }
-      else
-        {
-          g_string_append_c (string, *p);
-        }
-    }
-
-  return g_string_free (string, FALSE);
-}
-
-
-
 /* Migration for history cache file, can be removed during the 4.20 cycle */
 static void
 xfce_appfinder_model_migrate_history_file (void)
@@ -2576,16 +2581,11 @@ xfce_appfinder_model_execute (XfceAppfinderModel  *model,
                                                    garcon_menu_item_requires_terminal (item));
   g_free (uri);
 
-#if LIBXFCE4UTIL_CHECK_VERSION (4, 18, 0)
   escaped_command = xfce_unescape_desktop_entry_value (command);
-#else
-  escaped_command = xfce_appfinder_model_unescape_entry_value (command);
-#endif
   g_free (command);
   command = escaped_command;
   escaped_command = NULL;
 
-#if GARCON_CHECK_VERSION (4, 17, 0)
   if (garcon_menu_item_get_prefers_non_default_gpu (item))
     {
       envp = g_get_environ ();
@@ -2593,26 +2593,15 @@ xfce_appfinder_model_execute (XfceAppfinderModel  *model,
       envp = g_environ_setenv (envp, "__GLX_VENDOR_LIBRARY_NAME", "nvidia", TRUE);
       envp = g_environ_setenv (envp, "__VK_LAYER_NV_optimus", "NVIDIA_only", TRUE);
     }
-#endif
 
   if (g_shell_parse_argv (command, NULL, &argv, error))
     {
-#if LIBXFCE4UI_CHECK_VERSION (4, 15, 6)
       succeed = xfce_spawn (screen,
                             garcon_menu_item_get_path (item),
                             argv, envp, G_SPAWN_SEARCH_PATH,
                             garcon_menu_item_supports_startup_notification (item),
                             gtk_get_current_event_time (),
                             icon, TRUE, error);
-#else
-      succeed = xfce_spawn_on_screen (screen,
-                                      garcon_menu_item_get_path (item),
-                                      argv, envp, G_SPAWN_SEARCH_PATH,
-                                      garcon_menu_item_supports_startup_notification (item),
-                                      gtk_get_current_event_time (),
-                                      icon, error);
-#endif
-
       g_strfreev (argv);
     }
 
@@ -2639,7 +2628,7 @@ xfce_appfinder_model_load_pixbuf (const gchar           *icon_name,
     {
     case XFCE_APPFINDER_ICON_SIZE_SMALLEST: size = 16;  break;
     case XFCE_APPFINDER_ICON_SIZE_SMALLER:  size = 24;  break;
-    case XFCE_APPFINDER_ICON_SIZE_SMALL:    size = 36;  break;
+    case XFCE_APPFINDER_ICON_SIZE_SMALL:    size = 32;  break;
     case XFCE_APPFINDER_ICON_SIZE_NORMAL:   size = 48;  break;
     case XFCE_APPFINDER_ICON_SIZE_LARGE:    size = 64;  break;
     case XFCE_APPFINDER_ICON_SIZE_LARGER:   size = 96;  break;
@@ -2897,6 +2886,40 @@ xfce_appfinder_model_icon_theme_changed (XfceAppfinderModel *model)
 
 
 
+void
+xfce_appfinder_model_generic_names_changed (XfceAppfinderModel *model)
+{
+  ModelItem   *item;
+  GtkTreeIter  iter;
+  GtkTreePath *path;
+  gint         idx;
+  GSList      *li;
+
+  appfinder_return_if_fail (XFCE_IS_APPFINDER_MODEL (model));
+
+  APPFINDER_DEBUG ("item generic name changed, updating %d items",
+                   g_slist_length (model->items));
+
+  /* update model items */
+  for (li = model->items, idx = 0; li != NULL; li = li->next, idx++)
+    {
+      item = li->data;
+
+      if (item->item != NULL)
+        {
+          g_free (item->abstract);
+          item->abstract = NULL;
+
+          path = gtk_tree_path_new_from_indices (idx, -1);
+          ITER_INIT (iter, model->stamp, li);
+          gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
+          gtk_tree_path_free (path);
+        }
+    }
+}
+
+
+
 void
 xfce_appfinder_model_history_clear (XfceAppfinderModel *model)
 {
