@@ -124,6 +124,7 @@ static gint       xfce_appfinder_window_sort_items_frecency           (GtkTreeMo
                                                                        GtkTreeIter                 *b,
                                                                        gpointer                     data);
 static gchar    **xfce_appfinder_parse_envp                           (gchar                      **cmd);
+static guint      xfce_appfinder_get_navigation_key                   (GdkEventKey                 *event);
 
 struct _XfceAppfinderWindowClass
 {
@@ -177,6 +178,12 @@ static const GtkTargetEntry target_list[] =
   { "text/uri-list", 0, 0 }
 };
 
+enum {
+  XFCE_APPFINDER_KEY_NONE,
+  XFCE_APPFINDER_KEY_NEXT,
+  XFCE_APPFINDER_KEY_PREVIOUS
+};
+
 
 
 G_DEFINE_TYPE (XfceAppfinderWindow, xfce_appfinder_window, GTK_TYPE_WINDOW)
@@ -261,6 +268,7 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
                                                         XFCE_APPFINDER_ICON_SIZE_DEFAULT_ITEM,
                                                         scale_factor);
   window->image = image = gtk_image_new_from_pixbuf (window->icon_find);
+  gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (image)), "application-icon");
   gtk_widget_set_size_request (image, 48, 48);
   gtk_widget_set_halign(image, GTK_ALIGN_CENTER);
   gtk_container_add (GTK_CONTAINER (hbox), image);
@@ -1013,7 +1021,7 @@ xfce_appfinder_window_popup_menu_revert (GtkWidget           *mi,
   if (name == NULL)
     return;
 
-  if (xfce_dialog_confirm (GTK_WINDOW (window), XFCE_APPFINDER_ICON_NAME_REVERT, NULL,
+  if (xfce_dialog_confirm (GTK_WINDOW (window), XFCE_APPFINDER_ICON_NAME_REVERT, _("Confirm"),
           _("This will permanently remove the custom desktop file from your home directory."),
           _("Are you sure you want to revert \"%s\"?"), name))
     {
@@ -1476,9 +1484,8 @@ xfce_appfinder_window_entry_key_press_event (GtkWidget           *entry,
                                              GdkEventKey         *event,
                                              XfceAppfinderWindow *window)
 {
-  gboolean          expand;
-  gboolean          is_expanded;
-  GtkTreePath      *path;
+  GtkTreePath *path;
+  guint        navigation_key;
 
   if (event->keyval == GDK_KEY_Tab &&
       !gtk_widget_get_visible (window->paned) &&
@@ -1488,39 +1495,48 @@ xfce_appfinder_window_entry_key_press_event (GtkWidget           *entry,
       return TRUE;
     }
 
-  if (event->keyval == GDK_KEY_Up ||
-      event->keyval == GDK_KEY_Down)
+  navigation_key = xfce_appfinder_get_navigation_key(event);
+
+  if (navigation_key == XFCE_APPFINDER_KEY_NONE)
+    return FALSE;
+
+  if (!gtk_widget_get_visible (window->paned)) /* collapsed mode */
     {
-      expand = (event->keyval == GDK_KEY_Down);
-      is_expanded = gtk_widget_get_visible (window->paned);
-
-      if (is_expanded != expand)
+      /* don't break entry completion navigation in collapsed mode */
+      if (xfce_appfinder_window_pointer_is_grabbed (entry))
         {
-          /* don't break entry completion navigation in collapsed mode */
-          if (!is_expanded && xfce_appfinder_window_pointer_is_grabbed (entry))
-            {
-              /* window is still collapsed and the pointer is grabbed
-               * by the popup menu, do nothing with the event */
-              return FALSE;
-            }
+          /* the pointer is grabbed by the completion menu, ignore the event */
+          return FALSE;
+        }
 
-          xfce_appfinder_window_set_expanded (window, expand);
+      if (navigation_key == XFCE_APPFINDER_KEY_NEXT)
+        {
+          xfce_appfinder_window_set_expanded (window, TRUE);
           return TRUE;
         }
 
-      /* The first item is usually selected, so we should jump to 2nd one
-       * when down key is pressed */
-      if (is_expanded && expand &&
-          GTK_IS_TREE_VIEW (window->view) && /* does not make sense for icon view */
+      return FALSE;
+    }
+
+  /* expanded mode */
+
+  if (navigation_key == XFCE_APPFINDER_KEY_NEXT)
+    {
+      /* if the first item is selected, we should jump to 2nd one when next key is pressed */
+      if (GTK_IS_TREE_VIEW (window->view) && /* makes sense for list view, not icon view */
           xfce_appfinder_window_view_get_selected_path (window, &path))
         {
           gtk_tree_path_next (path);
           gtk_tree_view_set_cursor (GTK_TREE_VIEW (window->view), path, NULL, FALSE);
           gtk_tree_path_free (path);
         }
+
+      gtk_widget_grab_focus (window->view);
+      return TRUE;
     }
 
-  return FALSE;
+  xfce_appfinder_window_set_expanded (window, FALSE);
+  return TRUE;
 }
 
 
@@ -1580,7 +1596,8 @@ xfce_appfinder_window_treeview_key_press_event (GtkWidget           *widget,
                                                 XfceAppfinderWindow *window)
 {
   GdkEvent     ev;
-  GtkTreePath *path;
+  GtkTreePath *path = NULL;
+  guint        navigation_key;
 
   if (widget == window->view)
     {
@@ -1590,8 +1607,7 @@ xfce_appfinder_window_treeview_key_press_event (GtkWidget           *widget,
           event->keyval == GDK_KEY_Shift_R ||
           event->keyval == GDK_KEY_Alt_L ||
           event->keyval == GDK_KEY_Alt_R ||
-          event->keyval == GDK_KEY_Right ||
-          event->keyval == GDK_KEY_Down)
+          event->keyval == GDK_KEY_Right)
           return FALSE;
 
       if (event->keyval == GDK_KEY_Left)
@@ -1601,15 +1617,31 @@ xfce_appfinder_window_treeview_key_press_event (GtkWidget           *widget,
           return TRUE;
         }
 
-      if (event->keyval == GDK_KEY_Up)
+      xfce_appfinder_window_view_get_selected_path (window, &path);
+      navigation_key = xfce_appfinder_get_navigation_key (event);
+
+      if (path == NULL || navigation_key == XFCE_APPFINDER_KEY_NONE)
         {
-          if (xfce_appfinder_window_view_get_selected_path (window, &path))
+          /* do nothing */
+        }
+      else if (navigation_key == XFCE_APPFINDER_KEY_NEXT)
+        {
+          gtk_tree_path_next (path);
+          gtk_tree_view_set_cursor (GTK_TREE_VIEW (window->view), path, NULL, FALSE);
+          gtk_tree_path_free (path);
+          return TRUE;
+        }
+      else if (navigation_key == XFCE_APPFINDER_KEY_PREVIOUS)
+        {
+          if (gtk_tree_path_prev (path))
             {
-              if (!gtk_tree_path_prev (path))
-                gtk_widget_grab_focus (window->entry);
+              gtk_tree_view_set_cursor (GTK_TREE_VIEW (window->view), path, NULL, FALSE);
               gtk_tree_path_free (path);
+              return TRUE;
             }
 
+          gtk_widget_grab_focus (window->entry);
+          gtk_tree_path_free (path);
           return FALSE;
         }
 
@@ -2250,3 +2282,26 @@ xfce_appfinder_parse_envp (gchar **cmd)
 
   return envp;
 }
+
+
+
+static guint
+xfce_appfinder_get_navigation_key (GdkEventKey *event)
+{
+  if (event->keyval == GDK_KEY_Down)
+    return XFCE_APPFINDER_KEY_NEXT;
+
+  if (event->keyval == GDK_KEY_Up)
+    return XFCE_APPFINDER_KEY_PREVIOUS;
+
+  if ((event->state & GDK_CONTROL_MASK) != 0)
+    {
+      if (event->keyval == GDK_KEY_n || event->keyval == GDK_KEY_N)
+        return XFCE_APPFINDER_KEY_NEXT;
+
+      if (event->keyval == GDK_KEY_p || event->keyval == GDK_KEY_P)
+        return XFCE_APPFINDER_KEY_PREVIOUS;
+    }
+
+  return XFCE_APPFINDER_KEY_NONE;
+}
